#!/bin/tcsh
####################################################################################
#Author: Zhang Tao
#Email: zhangtao@cryst.iphy.ac.cn
#Created time: 2008,05,20
#last Edit: 2008,05,20
#Group: SM5,  The Institute of Physics, Chinese Academy of Sciences
#####################################################################################

######parameters##################################################
#1: parameters pool
#2: number of cycles
#3: main states file
#4: chang parameters pool
#5: number of seed
#6: interface file name
#7: last parameters pool
###################################################################################

source $1
set name_c = ${name}_$2_$5
set stat_file = .par_tmp/${name_c}.sta


if ( -e $stat_file ) then
source $4
source $stat_file
goto $program
else
touch $stat_file
echo "set stat_file = $stat_file" >> $4
echo "set name_c = $name_c " >> $4
echo "set html_name = $html_name" >> $4
endif
echo "set mr_states = 0 " >> $3
echo "set mr_states = 0 " >> $stat_file

if ( $RFREE != 'Unassigned' ) then
  echo "set hklin_free = $oasis_hklin" >> $4
endif

echo "set OASIS_MODE = DMR " >> $4
if ( $#argv == 6  ) then
  echo "set oasis_hklin = $oasis_hklin "   >> $4
  if ( ${PAR} == '1' && ${phase_signal} == "oasis" ) then
    cp $oasis_frc ./${name_c}_partial.frc
   endif
else
  source $7
  cp $build_frc_out ./${name_c}_partial.frc
  if ( $build_fp != null && $build_sigfp != null  ) then  
    echo "set oasis_hklin =  $build_mtz_out " >> $4
    echo "set FP = $build_fp " >> $4
    echo "set SIGFP = $build_sigfp" >> $4
  else
    echo "set oasis_hklin = $oasis_hklin "   >> $4
  endif
endif
echo "set oasis_frc = ./${name_c}_partial.frc" >> $4
set name_c = ${name}_$2_$5
set stat_file = .par_tmp/${name_c}.sta


echo "set init_signal = 0" >> $4
echo "set fom_signal = $fom_signal" >> $4
if ( $fom_signal == 1 ) then
	echo "set fom_file = $fom_file" >> $4
	echo "set FOMM = $FOMM" >> $4
endif
echo "set fof_signal = $fof_signal" >> $4
echo "set hl_control_signal = $hl_control_signal" >> $4
if ( $hl_control_signal == 1 ) then
  if ( $?dock_residue_ratio == 1 ) then
    set hl_oasis_signal = `echo $dock_residue_ratio  $hl_ratio | awk '{ if ( $1 < $2 ) print 1;else print 0; }' `
  else 
    set hl_oasis_signal = 1 
  endif
  if ( $hl_oasis_signal == 1 ) then
    echo "set hl_signal = $hl_signal"  >> $4
    if ( $hl_signal == 1 ) then
      echo "set hl_file = $hl_file" >> $4
      echo "set HLA = $HLA" >> $4
      echo "set HLB = $HLB" >> $4
      echo "set HLC = $HLC" >> $4
      echo "set HLD = $HLD" >> $4
    endif
    echo "set hl_oasis_signal = 1"  >> $4
  else
    echo "set hl_signal = 0" >> $4
    echo "set hl_oasis_signal = 0"  >> $4
  endif
else
  echo "set hl_signal = 0" >> $4
  echo "set hl_oasis_signal = 0"  >> $4
endif


echo "set sigma_signal = $sigma_signal" >> $4
if ( $sigma_signal == 1  ) then
  echo "set sigma_file = $sigma_file" >> $4
endif
echo "set ncs_signal = $ncs_signal" >> $4
if ( $ncs_signal == 1 ) then	
	echo "set ncs_file = $ncs_file" >> $4
endif



#execute oasis
phase:
echo "set program = phase" >> $stat_file
if ( $phase_signal == "oasis" || $2 > 0 ) then
echo "set phase_log = ${name_c}_oasis.log"  >> $4
${oasisbin}/oasis_exe.csh $1 $4 $stat_file $2 $5 | tee ${name_c}_oasis.log
else if ( $phase_signal == "phaser" || $2 == 0  ) then
echo "set phase_log = ${name_c}_phaser.log"  >> $4
${oasisbin}/phaser_exe.csh $1 $4 $stat_file $2  | tee ${name_c}_phaser.log
endif
source $stat_file
if ( $phase_status == 0 ) goto exitmr
if ( -e .par_tmp/oasis.fin ) goto stop

#execute dm 
dm:
if ( $if_dm_signal == "1" ) then
  if (  $dm_signal == "dm"  ) then
  echo "set program = dm" >> $stat_file
  echo "set dm_log = ${name_c}_dm.log"  >> $4
  ${oasisbin}/dm_exe.csh $1 $4 $stat_file | tee ${name_c}_dm.log
  else if  (  $dm_signal == "resolve"  ) then
  echo "set program = dm" >> $stat_file
  echo "set dm_log = ${name_c}_dm_resolve.log"  >> $4
  ${oasisbin}/resolve_dm_exe.csh $1 $4 $stat_file | tee ${name_c}_dm_resolve.log
  endif
  source $stat_file
  echo "set if_dm = 1" >> $4
  if ( $dm_status == 0 ) goto exitmr
  if ( -e .par_tmp/oasis.fin ) goto stop
else
  echo "set if_dm = 0" >> $4
endif



#execute merge 
merge:
if ( -e .par_tmp/oasis_${2}.mer ) then
  echo "set program = merge" >> $stat_file
  ${oasisbin}/merge_exe.csh $1 $4 $stat_file .par_tmp/oasis.mer
  source $stat_file
  if ( $merge_status == 0 ) then
    echo "set merge_signal = fail"
  else
    echo "set merge_signal = success"
  endif
else
  echo "set merge_signal = null"
endif

#execute build program
build:
echo "set program = build" >> $stat_file
if ( $build_mod == "resolve" || $build_mod == "auto" && $2 < $auto_build_num ) then
  echo "set build_log = ${name_c}_resolve.log"  >> $4
  echo "set select_mode = $mr_resolve_select_mode " >> $4
  ${oasisbin}/resolve_build_exe.csh $1 $4 $stat_file | tee ${name_c}_resolve.log
else if ( $build_mod == "arp" || $build_mod == "auto" && $2 >= $auto_build_num ) then
  echo "set build_log = ${name_c}_arp.log"  >> $4
  echo "set select_mode = $mr_arp_select_mode " >> $4
  ${oasisbin}/arpwarp_exe.csh $1 $4 $stat_file | tee ${name_c}_arp.log
else if ( $build_mod == "phenix" ) then
  echo "set build_log = ${name_c}_phenix.log"  >> $4
  echo "set select_mode = $mr_phenix_select_mode " >> $4
  ${oasisbin}/phenix_build_exe.csh $1 $4 $stat_file | tee  ${name_c}_phenix.log
endif
source $stat_file
if ( $build_status == 0 ) goto exitmr
if ( -e .par_tmp/oasis.fin ) goto stop




#register information
source $4
#if ( $build_pro == "resolve" ) then
#echo "${OASIS_MODE} resolve $2 $recommand[1] $recommand[2] $recommand[3] $5 " >> $html_name/data.log
#else
#echo "${OASIS_MODE} arp $2 $recommand[1] $recommand[2] $recommand[3] $dock_best[1] $dock_best[2] $dock_best[3] $res_best[1] #$res_best[2] $res_best[3] $r_best[1] $r_best[2] $r_best[3] $5 " >> $html_name/data.log
#endif



#show information
${oasisbin}/html/show.csh $4 $stat_file $2 $6 1 $5

echo "set mr_states = 1 " >> $3
echo "set mr_states = 1 " >> $stat_file
exit


exitmr:
echo "set mr_states = 0 " >> $3
${oasisbin}/html/show.csh $4 $stat_file $2 $6 0 $5
source $4
if ( $program == "phase" ) then
echo "set error_log = $phase_log" >> $3
else if ( $program == "dm" ) then
echo "set error_log = $dm_log" >> $3
else if ( $program == "build" ) then
echo "set error_log = $build_log" >> $3
endif

stop:
echo "set sad_states = 1 " >> $stat_file
echo "set sad_states = 1 " >> $3

