#
#     Copyright (C) 1999-2004  Liz Potterton, Peter Briggs
#
#     This code is distributed under the terms and conditions of the
#     CCP4 Program Suite Licence Agreement as a CCP4 Library.
#     A copy of the CCP4 licence can be obtained by writing to the
#     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
#
#CCP4i_cvs_Id $Id: oasis.tcl,v 1.9 2005/09/09 09:40:33 mdw Exp $
# ======================================================================
# oasis.tcl --
#
# CCP4Interface 
#
# =======================================================================

#-----------------------------------------------------------------------
proc oasis_run { arrayname } {
#-----------------------------------------------------------------------
  upvar #0 $arrayname array

  # If DM is going to run then check for a valid solvent content
    if { $array(DM) || $array(if_dm_signal) } {
	if { $array(solc) <= 0.0 || $array(solc) >= 1.0 } {
	    WarningMessage "You have specified an invalid solvent fraction"
	    return 0
	}
    }

  if { $array(KMI) <= 0.0  && $array(KMI) != "" && $array(KMI) != "auto"   } {
    WarningMessage "You have specified an invalid Kappa minimum"
    return 0
  }





  if { [file exists [GetFullFileName1 $array(HKLIN) $array(DIR_HKLIN) ] ] } {
    set array(INPUT_FILES) " HKLIN "		
  } else {
    WarningMessage "Can not open MTZ input file." -button Cancel
    return 0
  }


  if { [GetValue $arrayname MOD] == 1 } {
    if { [file exists [GetFullFileName1 $array(M_HKLIN) $array(DIR_M_HKLIN) ] ] } {
      set array(INPUT_FILES) " M_HKLIN "		
    } else {
      WarningMessage "Can not open FOM MTZ input file." -button Cancel
      return 0
    }
    append array(INPUT_FILES) " M_HKLIN"
  }

  if { [GetValue $arrayname DM] == 1  } {
    set NAME [split $array(HKLOUT) .]
    set HKL_T [lindex $NAME 0]
    append HKL_T "_dm.mtz"
    set array(DM_HKLOUT) $HKL_T
    set array(DIR_DM_HKLOUT) $array(DIR_HKLOUT)
    append array(OUTPUT_FILES) " DM_HKLOUT"
  }

    if { [GetValue $arrayname PAR] == 1 && [GetValue $arrayname PAR_MODE] == "P_PDB" } {
      if { [file exists [GetFullFileName1 $array(PAR_PDB_FILE) $array(DIR_PAR_PDB_FILE) ] ] } {
        set array(INPUT_FILES) " PAR_PDB_FILE "		
      } else {
        WarningMessage "Can not open partial structure PDB file." -button Cancel
        return 0
      }
    }

#    if { [GetValue $arrayname C_STD] == 1 } {
#	append array(INPUT_FILES) " STD_FILE"
#    }

    if { [GetValue $arrayname PAR] == 1 && [GetValue $arrayname PAR_MODE] == "P_OAS" } {
      if { [file exists [GetFullFileName1 $array(PAR_OAS_FILE) $array(DIR_PAR_OAS_FILE) ] ] } {
        set array(INPUT_FILES) " PAR_OAS_FILE "		
      } else {
        WarningMessage "Can not open partial structur file." -button Cancel
        return 0
      }
    }


    if { [GetValue $arrayname HA] == "HA_PDB" && [GetValue $arrayname HA_SIG] == 1  }  {
      if { [file exists [GetFullFileName1 $array(HA_FILE_PDB) $array(DIR_HA_FILE_PDB) ] ] } {
        set array(INPUT_FILES) " HA_FILE_PDB "		
      } else {
        WarningMessage "Can not open heavy atom PDB file." -button Cancel
        return 0
      }
    }

    if { [GetValue $arrayname HA] == "HA_OAS" && [GetValue $arrayname HA_SIG] == 1 }  {
      if { [file exists [GetFullFileName1 $array(HA_FILE_OAS) $array(DIR_HA_FILE_OAS) ] ] } {
        set array(INPUT_FILES) " HA_FILE_OAS "		
      } else {
        WarningMessage "Can not open heavy atom file." -button Cancel
        return 0
      }
    }

    if { [GetValue $arrayname CELL_CON] == "FILE" } {
      if { [file exists [GetFullFileName1 $array(SEQIN) $array(DIR_SEQIN) ] ] } {
        set array(INPUT_FILES) " SEQIN "		
         } else {
        WarningMessage "Can not open sequence file." -button Cancel
        return 0
      }
          } elseif { [GetValue $arrayname CELL_CON] == "OFILE" } {
      if { [file exists [GetFullFileName1 $array(OSEQIN) $array(DIR_OSEQIN) ] ] } {
        set array(INPUT_FILES) " OSEQIN "
         } else {
        WarningMessage "Can not open sequence file." -button Cancel
        return 0
      }
     }


    if { [GetValue $arrayname CYC_SIG] == 1 } {
      if { [file exists [GetFullFileName1 $array(WORK_DIR) $array(DIR_WORK_DIR) ] ]  } {
        set action [ChooseOptionDialog "Old job exist" "Old job exist" " There have been a job existed in work directory you select. It will be removed." { Abort Continue } -default 1]
        if [regexp Abort $action ] { return 0 }
      }
      set wdir_list [split [GetFullFileName1 $array(WORK_DIR) $array(DIR_WORK_DIR) ] "/"]
      set wdir "/"
      for { set i 1 } { $i <=  [llength $wdir_list] - 2  } { incr i } { 
        if { [lindex $wdir_list $i] != "" } {
          append wdir [lindex $wdir_list $i] 
          append wdir "/"
        }
      }
      if { [file exists $wdir] == 0 } {
        WarningMessage "Can not create the work directory." -button Cancel
        return 0
      }
      
      if { [GetValue $arrayname NCS] == 1 } {
        if { [file exists [GetFullFileName1 $array(NCS_FILE) $array(DIR_NCS_FILE) ] ] } {
          set array(INPUT_FILES) " NCS_FILE "		
        } else {
          WarningMessage "Can not open NCS file." -button Cancel
          return 0
        }
      }
      if { [GetValue $arrayname HL_control_signal] && ( [GetValue $arrayname HL_ratio] <= 0 || [GetValue $arrayname HL_ratio] >= 100 ) } {
        set action [ChooseOptionDialog "invalid data" "invalid data" " HL tatio is invalid, please check it " { Abort Continue } -default 1]
        if [regexp Abort $action ] { return 0 }
      }
      if { ( [GetValue $arrayname AUTO_BUILD_NUM] <= 0 || [GetValue $arrayname AUTO_BUILD_NUM] >= [GetValue $arrayname CYC_NUM] ) && [GetValue $arrayname BUILD_TYPE] == "auto"  } {
        set action [ChooseOptionDialog "invalid data" "invalid data" " Auto building number is invalid, please check it " { Abort Continue } -default 1]
        if [regexp Abort $action ] { return 0 }
      }
    if { [GetValue $arrayname skip_num] >= [GetValue $arrayname SED] && $array(FIRST_SIG)==0 || [GetValue $arrayname skip_num] > [GetValue $arrayname first_seed] && $array(FIRST_SIG)  } {
        WarningMessage "Skip trials should be smaller than number of trials at first cycle." -button Cancel
        return 0
    }
    } else {
      set array(OUTPUT_FILES) "HKLOUT"
    }

  if { [GetValue $arrayname OASIS_MODE] == "SAD" } {
    if { [GetValue $arrayname MAP_FM] } {
      if { $array(FP) == "Unassigned" || $array(SIGFP) == "Unassigned"  || \
           $array(DANO) == "Unassigned" || $array(SIGDANO) == "Unassigned" }  {
          WarningMessage "NOT all data are assigned to an mtz label." -button Cancel
          return 0
      }
    } elseif { [GetValue $arrayname MAP_FP] } {
      if { $array(FPP) == "Unassigned" || $array(SIGFPP) == "Unassigned"  || \
           $array(FPN) == "Unassigned" || $array(SIGFPN) == "Unassigned" }  {
          WarningMessage "NOT all data are assigned to an mtz label." -button Cancel
          return 0
      }
    }
    if { [GetValue $arrayname X_RAY ] != "X_WL" } {
      for {set i 1} { $i <= $array(HA_T_NUM)} {incr i} {
         if { $array(X_F,$i) == "" || $array(X_F,$i) <= 0 } {
           WarningMessage "You should input  f'' about SAD." -button Cancel
           return 0
         }
      }
    } else {
      if { [GetValue $arrayname X_WLENGTH ] == "" || [GetValue $arrayname X_WLENGTH ] <= 0 } {
           WarningMessage "You should input wavalength about SAD." -button Cancel
           return 0
       }
    }
  }

  if { [GetValue $arrayname OASIS_MODE] == "SIR" } {
    if { $array(FP) == "Unassigned" || $array(SIGFP) == "Unassigned" || \
       $array(FPH) == "Unassigned" || $array(SIGFPH) == "Unassigned" }  {
       WarningMessage "NOT all data are assigned to an mtz label." -button Cancel
       return 0
    }
  }

  if { [GetValue $arrayname OASIS_MODE] == "DMR" } {
    if { $array(FP) == "Unassigned" || $array(SIGFP) == "Unassigned"  }  {
        WarningMessage "NOT all data are assigned to an mtz label." -button Cancel
        return 0
    }
    if { [GetValue $arrayname NHA] >= 100 || [GetValue $arrayname NHA] <= 0 } {
        WarningMessage "Artificial heavy atoms ratio is invalid, the value should be between 1 and 100." -button Cancel
        return 0
    }
    if { [GetValue $arrayname SED] <= 0 } {
        if { [GetValue $arrayname CYC_SIG] } {
          WarningMessage "Number of trials is invalid, the value should be larger than 0." -button Cancel
          return 0
        } else {
          WarningMessage "Seed of random-number generator is invalid, the value should be larger than 0." -button Cancel
          return 0
        }
    }
  }

    if { [GetValue $arrayname HA] == "HA_EN" && [GetValue $arrayname HA_SIG] == 1 }  {
      set n 0
      for {set i 1} { $i <= $array(HA_T_NUM)} {incr i} {
        for {set j 1} { $j <= $array(HA_NUM_P,$i) } {incr j} {
          incr n
            set array(HA_TYPE,$n) $array(HA_T_TYPE,$i)
            set array(HA_X,$n) $array([Indxv HA_X_T $i $j])
            set array(HA_Y,$n) $array([Indxv HA_Y_T $i $j])
            set array(HA_Z,$n) $array([Indxv HA_Z_T $i $j])
            set array(HA_OCC,$n) $array([Indxv HA_OCC_T $i $j])
            set array(HA_B,$n) $array([Indxv HA_B_T $i $j])
       }
    }
    set array(HA_NUM) $n
  } 
  ha_up  $arrayname
  ha_c_p $arrayname
  c_up $arrayname

  return 1
}


#---------------------------------------------------------------------
proc oasis_setup { typedefVar arrayname } {
#---------------------------------------------------------------------
  upvar #0  $typedefVar typedef
  upvar #0 $arrayname array

    DefineMenu _oasis_mode [list "SAD --- direct phasing and fragment extension" \
	                         "SIR --- direct phasing and fragment extension"\
                                 "Partial-model extension without SAD/SIR information"] \
	                     [list SAD SIR DMR]


    DefineMenu _cell_con [list "number of residues in asymmetric unit" "from sequence file (*.pir)" "from other sequence file (*.*)"   ] \
	                        [list NUM FILE OFILE ]

    DefineMenu _cell_con_cyc [list "from sequence file (*.pir)" "from other sequence file (*.*)"   ] \
                                [list FILE OFILE ]


    DefineMenu _oasis_ha_entry [list "enter below" "from PDB file" "from Oasis file"] \
	    [list HA_EN HA_PDB HA_OAS ]

    DefineMenu _oasis_xray [list "X-ray wavelength" "input f''" ] \
	    [list X_WL X_F ]

 DefineMenu _par_mode [list "From PDB file" " From Oasis file" ] \
	    [list P_PDB P_OAS  ]

 DefineMenu _modify_mode [list  "merge" "Change FOMs" ] \
	    [list M_MER M_FOM  ]

 DefineMenu _fom_type [list  "OASIS" "Partial structure" ] \
	    [list OASIS PAR  ]

 DefineMenu _hl_type [list  "OASIS" "other file" ] \
	    [list OASIS OTHER  ]

 DefineMenu _ref_mode [list  "Maximum Likelihood" "Phased ML" ] \
	    [list REFML PHASED  ]


 DefineMenu _build_mod [list "  RESOLVE+ARP/wARP" "  AWP/wARP" "  RESOLVE" "  PHENIX.AutoBuild   " ] \
	    [list auto arp resolve  phenix ]

 DefineMenu _ext_mod [list  "  AWP/wARP"  "  PHENIX.AutoBuild   " ] \
	    [list arp  phenix ]

 DefineMenu _phase_mod_sad [list " OASIS " " PHASER " ] \
	    [list oasis phaser ]

 DefineMenu _phase_mod_mr [list " OASIS " " PHASER(MR) " ] \
	    [list oasis phaser ]


 DefineMenu _dm_mod [list " DM" " RESOLVE" ] \
	    [list dm resolve ]

 DefineMenu _arp_select_mod [list "default" "number of residues found" "number of residues docked" "best R factor" "best Rfree factor" ] \
	    [list normal cbest dbest rbest rfree ]

 DefineMenu _phenix_select_mod [list "default"  "Number of residues found" "residues placed" "best R factor" "best Rfree factor" "map-model CC" ] \
	    [list normal  cbest dbest rbest rfree score ]

 DefineMenu _mr_arp_select_mod [list "Number of residues found" "residues docked" "best R factor" "best Rfree factor" ] \
	    [list  cbest dbest rbest rfree  ]

 DefineMenu _mr_resolve_select_mod [list   "Number of residues found" "residues docked" "RESOLVE score" ] \
	    [list  cbest dbest score  ]

 DefineMenu _autobuild_mod [list   "auto" "customize..." ] \
	    [list  auto fix  ]

 DefineMenu _mr_phenix_select_mod [list   "map-model CC"  "Number of residues found" "residues docked" "R factor" ] \
	    [list score cbest dbest rbest ]
 DefineMenu _scale [list "SIMPLE" "BULK"  ]  [list simple bulk ]

 set      typedef(_dir)        { file DIR " "  "All" "" "" }
  return 1
}

# procedure to draw task window
#---------------------------------------------------------------------
proc oasis_task_window { arrayname } {
#---------------------------------------------------------------------
  upvar #0 $arrayname array


  if { [CreateTaskWindow $arrayname  \
        "Direct methods of SAD/SIR phasing and partial-model extension" "OASIS" \
        [ list "Heavy-atom substructure" \
               "Controlling parameters for partial-model extension"\
               "*** Cell Contents for calculating E values in direct-method phasing" \
               "Controlling parameters for direct-method phasing"   \
               "ARP/wARP parameters"   \
               "PHENIX.Autobuild parameters"   \

	] ] == 0 } return


# Set the name of the CCP4 program help file to use
  SetProgramHelpFile "oasis"

#=PROTOCOL==============================================================

  OpenFolder protocol 

  CreateTitleLine line TITLE
OpenSubFrame frame -toggledisplay OASIS_MODE open [list SAD SIR]
  CreateLine line \
	  message "Select one of the available actions to perform" \
	  label "Running mode " \
	  widget OASIS_MODE \
        -command "mode_p $arrayname" 
CloseSubFrame


OpenSubFrame frame -toggledisplay OASIS_MODE open [list DMR]
  CreateLine line \
	  message "Select one of the available actions to perform" \
	  label "Running mode " \
	  widget OASIS_MODE \
        -command "mode_p $arrayname" \
        widget SAD_TO_MR -command "sad_to_mr_p $arrayname" \
        label " Heavy atom " 
        bind $line.cb3 <Button-3>  "ShowMessage ha_mode"
        bind $line.l4 <Button-3>  "ShowMessage ha_mode"

CloseSubFrame

#=FILES================================================================ 

  OpenFolder file

  # Input MTZ
  CreateInputFileLine line \
        "Enter name of input mtz file" \
        "MTZ in  " \
        HKLIN DIR_HKLIN \
	-setfileparam space_group_name SPACEGROUP \
      -setfileparam resolution_min EXCLUDE_RESOLUTION_MIN \
      -setfileparam resolution_max EXCLUDE_RESOLUTION_MAX \
      -setfileparam resolution_max DATA_RESOLUTION \
	-setfileparam CELL_1 CELL_A \
	-setfileparam CELL_2 CELL_B \
	-setfileparam CELL_3 CELL_C \
	-setfileparam CELL_4 CELL_ALPHA \
	-setfileparam CELL_5 CELL_BETA \
	-setfileparam CELL_6 CELL_GAMMA \
	-fileout HKLOUT DIR_HKLOUT "_oasis_" \
        -command "in_p $arrayname"
  CreateLabinLine line \
	"Assign structure factor amplitude and its sigma" \
	HKLIN "         Fp" FP {NULL} \
	-sigma "    SigFp" SIGFP {NULL}\
      -toggle_display OASIS_MODE open [list DMR]


#   Input labels for SAD
OpenSubFrame frame -toggledisplay OASIS_MODE open [list SAD]


          CreateLine line \
                label "       SAD data format  :"\
		widget MAP_FM  label  "Fmean, Dano       "  -command "map_p1 $arrayname"\
		widget MAP_FP  label "F+, F -" -command "map_p2 $arrayname" 


OpenSubFrame frame -toggledisplay MAP_FM open 1

  CreateLabinLine line \
	"Assign the mean of |F(+)| and |F( - )| and the corresponding sigma" \
	HKLIN "     Fmean" FP {NULL} \
	-sigma "SigFmean" SIGFP {NULL}\

  CreateLabinLine line \
	"Assign anomalous difference, |F(+)| - |F( - )|, and its sigma" \
	HKLIN "     Dano" DANO {NULL} \
	-sigma "SigDano" SIGDANO {NULL}\


CloseSubFrame

OpenSubFrame frame -toggledisplay MAP_FP open 1

  CreateLabinLine line \
	"Assign structure factor amplitude |F(+)| and its sigma" \
	HKLIN "      F+" FPP {NULL} \
	-sigma "     SigF+" SIGFPP {NULL}\

  CreateLabinLine line \
	"Assign structure factor amplitude |F( - )| and its sigma" \
	HKLIN "      F -" FPN {NULL} \
	-sigma "     SigF -" SIGFPN {NULL}\


CloseSubFrame

CloseSubFrame


# Input labels for SIR

  OpenSubFrame frame -toggledisplay OASIS_MODE open [list SIR]

   CreateLabinLine line \
	"For SIR: assign native structure factor amplitude and sigma" \
	HKLIN "           Fp" FP {NULL} \
	-sigma "      SigFp" SIGFP {NULL}

  CreateLabinLine line \
	"For SIR: assign isomorphous difference and sigma" \
	HKLIN "         Fph" FPH {NULL} \
	-sigma "    SigFph" SIGFPH {NULL} \

 CloseSubFrame

    CreateLabinLine line \
      "Choose Free R column data (LABIN FREER)" \
      HKLIN "     FreeR" FREE  {Unassigned}
 


  # Output MTZ
  CreateOutputFileLine line \
	"Enter name of output mtz file" \
       "MTZ out" \
	HKLOUT DIR_HKLOUT -toggle_display CYC_SIG open 0

  CreateOutputFileLine line \
	"Enter name of output mtz file" \
       "Work directory" \
	WORK_DIR DIR_WORK_DIR -toggle_display CYC_SIG open 1


     OpenSubFrame frame -toggle_display OASIS_MODE open [list  SAD SIR]

          CreateLine line \
            message "Right click for Help" \
            widget HL_control_signal \
            label " Use original unresolved bimodal phase distribution in DM, RESOLVE, Refmac and AutoBuild"
            bind $line.cb1 <Button-3>  "ShowMessage hl_opt"



     OpenSubFrame frame -toggle_display HL_control_signal open 1


         CreateLine line \
            message "Right click for Help" \
            label "      when assigned residues in the built model are no more than  " \
            widget HL_ratio \
            label "% of residues in the structure"
            bind $line.e2 <Button-3>  "ShowMessage hl_ratio"


#         CreateLine line \
#            label "                Input HL coefficients"\
#            widget HL_TYPE 
      

#    OpenSubFrame frame -toggle_display HL_TYPE open OTHER


#      CreateInputFileLine line \
#        "Enter name of input mtz file" \
#        "          MTZ in  " \
#        HL_HKLIN DIR_HL_HKLIN\
#        -setlabin HLA [list HLA HLAM ] \
#        -setlabin HLB [list HLB HLBM ] \
#        -setlabin HLC [list HLC HLCM ] \
#        -setlabin HLD [list HLD HLDM ] 

#     CreateLabinLine line \
#	  "Hendrickson-Lattmann coefficients"  \
#	 HL_HKLIN "        HLA" HLA  {  } \
#	  -sigma "         HLB" HLB { }

#    CreateLabinLine line \
#        "Hendrickson-Lattmann coefficients"  \
#        HL_HKLIN "        HLC" HLC { } \
#	  -sigma "         HLD" HLD { }

#   CloseSubFrame
   CloseSubFrame
   CloseSubFrame






    CreateLine line \
            message "Right click for Help" \
            widget CYC_SIG  label "Dual-space iteration " -command "cyc_p $arrayname" 
    bind $line.cb1 <Button-3>  "ShowMessage iteration_opt"


OpenSubFrame frame -toggle_display CYC_SIG open 1

     CreateLine line \
            message "Right click for Help" \
            label "           Maximum number of cycles"\
            widget CYC_NUM \

     bind $line.e2 <Button-3>  "ShowMessage iteration_num"

     OpenSubFrame frame -toggle_display OASIS_MODE open [list SAD SIR]

         CreateLine line \
            message "Right click for Help" \
            label "           Phasing by "  widget PHASE_TYPE_SAD -command "phase_type_p $arrayname"
         bind $line.mb2 <Button-3>  "ShowMessage phasing"

     CloseSubFrame

     OpenSubFrame frame -toggle_display OASIS_MODE open [list DMR]
         CreateLine line \
            message "Right click for Help" \
            label "           Phasing by "  widget PHASE_TYPE_MR -command "phase_type_p $arrayname" 
         bind $line.mb2 <Button-3>  "ShowMessage phasing"

     CloseSubFrame


     OpenSubFrame frame -toggle_display if_dm_signal open 1   
       CreateLine line \
            message "Right click for Help" \
            label "     " widget if_dm_signal label "Density modification by"\
            widget DM_MOD label "                  Solvent contents" \
            widget solc
         bind $line.cb2 <Button-3>  "ShowMessage dm"
         bind $line.mb3 <Button-3>  "ShowMessage dm_pro"
         bind $line.e5 <Button-3>  "ShowMessage solvent"
     CloseSubFrame

     OpenSubFrame frame -toggle_display if_dm_signal open 0   
       CreateLine line \
            message "Right click for Help" \
            label "     " widget if_dm_signal label "Density modification " 
         bind $line.cb2 <Button-3>  "ShowMessage dm"

     CloseSubFrame



    CreateLine line \
            message "Right click for Help" \
            label "            " widget NCS  label "NCS information"
         bind $line.cb2 <Button-3>  "ShowMessage ncs_opt"


OpenSubFrame frame -toggle_display NCS open 1

        CreateInputFileLine line \
             "Right click for Help" \
             "                 input file" \
	    NCS_FILE DIR_NCS_FILE 
        bind $line.mb2 <Button-3>  "ShowMessage ncs_file"
        bind $line.e3 <Button-3>  "ShowMessage ncs_file"
	CloseSubFrame 




    OpenSubFrame frame -toggle_display BUILD_TYPE open [list auto]

     CreateLine line \
            message "Right click for Help" \
            label "           Model building by "  widget BUILD_TYPE -command "build_type_p $arrayname"\
            label "       starting with "\
            widget AUTO_BUILD_NUM \
            label " cycles of RESOLVE  " 
            bind $line.mb2 <Button-3>  "ShowMessage build_mode"
            bind $line.e4 <Button-3>  "ShowMessage build_resolve_num"
	CloseSubFrame 

    OpenSubFrame frame -toggle_display BUILD_TYPE hide [list auto]
     CreateLine line  \
            message "Right click for Help" \
            label "           Model building by "  widget BUILD_TYPE -command "build_type_p $arrayname" 
            bind $line.mb2 <Button-3>  "ShowMessage build_mode"
	CloseSubFrame             





#CreateLine line \
	  message "apply Hendrickson-Lattmann coefficients" \
        label "       " \
	  widget A_HL \
	  label "apply Hendrickson-Lattmann coefficients" 


#OpenSubFrame frame -toggle_display A_HL open 1

#  CreateLabinLine line \
	"Hendrickson-Lattmann coefficients"  \
	HKLIN "       HLA" HLA  {} \
	-sigma "      HLB" HLB {}

# CreateLabinLine line \
        "Hendrickson-Lattmann coefficients"  \
        HKLIN "       HLC" HLC {} \
	-sigma "      HLD" HLD {}

#CloseSubFrame 



#CreateLine line \
	  message "Calculate phase error" \
        label "       " \
	  widget C_STD \
	  label "Calculate phase error" 
#OpenSubFrame frame -toggle_display C_STD open 1


#CreateInputFileLine line \
	    "Standard phase file" \
	    "       MTZ input" \
	    STD_FILE DIR_STD_FILE 

#CloseSubFrame 

    CloseSubFrame 


      CreateLine line \
            message "Right click for Help" \
            widget PAR  \
            label "Partial structure" -command "par_p $arrayname" 
      bind $line.cb1 <Button-3>  "ShowMessage partial"


OpenSubFrame frame -toggle_display PAR open 1




CreateLine line \
            message "Right click for Help" \
            label "     " widget PAR_MODE -command "par_format_p $arrayname"
     bind $line.mb2 <Button-3>  "ShowMessage partial_file"

CreateInputFileLine line \
                                  "Right click for Help" \
	    "               " \
	    PAR_PDB_FILE DIR_PAR_PDB_FILE \
	    -toggle_display PAR_MODE open [list P_PDB]
     bind $line.mb2 <Button-3>  "ShowMessage partial_file_pdb"
     bind $line.e3 <Button-3>  "ShowMessage partial_file_pdb"

CreateInputFileLine line \
                                  "Right click for Help" \
	    "               " \
	    PAR_OAS_FILE DIR_PAR_OAS_FILE \
	    -toggle_display PAR_MODE open [list P_OAS]
     bind $line.mb2 <Button-3>  "ShowMessage partial_file_frc"
     bind $line.e3 <Button-3>  "ShowMessage partial_file_frc"

  OpenSubFrame frame -toggle_display CYC_SIG open 1
  OpenSubFrame frame -toggle_display OASIS_MODE open DMR
  OpenSubFrame frame -toggle_display PHASE_TYPE_MR open oasis
    OpenSubFrame frame -toggle_display EXT_MODEL open 0
      CreateLine line \
            message "Right click for Help" \
            label "               " widget EXT_MODEL -command "par_format_p $arrayname"\
            label " Refine starting model "
            bind $line.cb2 <Button-3>  "ShowMessage partial_file_refine"


    CloseSubFrame
    OpenSubFrame frame -toggle_display EXT_MODEL open 1
      CreateLine line \
            message "Right click for Help" \
            label "               "  widget EXT_MODEL -command "par_format_p $arrayname"\
            label " Refine starting model      "   label " program"\
            widget EXT_PRO

            bind $line.cb2 <Button-3>  "ShowMessage partial_file_refine"
            bind $line.mb5 <Button-3>  "ShowMessage partial_file_refine_pro"

    CloseSubFrame
  CloseSubFrame
CloseSubFrame 
CloseSubFrame



 OpenSubFrame frame -toggle_display CYC_SIG open 0

     CreateLine line \
            message "Right click for Help" \
            label "       FOM out"\
            widget FOM_TYPE
     bind $line.l1 <Button-3>  "ShowMessage fom_from"
     bind $line.mb2 <Button-3>  "ShowMessage partial_fom"




  OpenSubFrame frame -toggle_display FOM_TYPE open PAR

  CreateInputFileLine line \
        "Enter name of input mtz file" \
        "       MTZ in  " \
        FOM_HKLIN DIR_FOM_HKLIN\
        -setlabin FOMM [list FOMDM FOM FOMM ] \
        -setlabin HLA [list HLA HLAM ] \
        -setlabin HLB [list HLB HLBM ] \
        -setlabin HLC [list HLC HLCM ] \
        -setlabin HLD [list HLD HLDM ]

     bind $line.l1 <Button-3>  "ShowMessage partial_mtz_in"
     bind $line.mb2 <Button-3>  "ShowMessage partial_fom_file"
     bind $line.e3 <Button-3>  "ShowMessage partial_fom_file"




   CreateLabinLine line \
            "Right click for Help" \
            FOM_HKLIN "         FOM" FOMM {NULL} 
            bind $line.mb2 <Button-3>  "ShowMessage partial_fom_select"
 CloseSubFrame



CloseSubFrame

CloseSubFrame
#  CreateLine line \
#	  widget MOD \
#	  label "Modify output phases"\

#OpenSubFrame frame -toggle_display MOD open 1
#    CreateLine line \
	   label "     " widget MOD_MOD

#OpenSubFrame frame -toggle_display MOD_MOD open M_MER

#  CreateInputFileLine line \
        "Enter name of input mtz file" \
        "       using the file" \
         M_HKLIN DIR_M_HKLIN

#  CreateLabinLine line \
	" " \
	M_HKLIN "       phase" M_PHIC {NULL} \
	-sigma "           FOM" M_FOM {NULL}

#    CreateLine line \
	   label "    "  widget  KEEP_MOD  label "keep unmodified MTZ file"

#CloseSubFrame 


#OpenSubFrame frame -toggle_display MOD_MOD open M_FOM

#  CreateInputFileLine line \
        "Enter name of input mtz file" \
        "       using the file" \
         M_HKLIN DIR_M_HKLIN

#    CreateLine line \
	   label "    "  widget  KEEP_MOD  label "keep unmodified MTZ file"

#CloseSubFrame 
#CloseSubFrame 


OpenSubFrame frame -toggle_display CYC_SIG open 0

    CreateLine line \
            message "Right click for Help" \
            widget DM  label "Density modifiction after OASIS " -command "dm_fom $arrayname" 
        bind $line.cb1 <Button-3>  "ShowMessage dm"


OpenSubFrame frame -toggle_display DM open 1
     CreateLine line \
     message "Right click for Help" \
     label "       Solvent contents"\
     widget solc 
     bind $line.e2 <Button-3>  "ShowMessage solvent"

 

#OpenSubFrame frame -toggle_display BUILD_SIG open [list 0 2]
#     CreateLine line \
#            label "       Solvent contents"\
#            widget solc  
#    CloseSubFrame 

    CloseSubFrame 
    CloseSubFrame 










#-----------------------------------------------------------------------------------------
 OpenFolder 1 HA_SIG  hide 0 open


    OpenSubFrame frame -toggle_display OASIS_MODE hide [list SAD]

       CreateLine line \
       message "Right click for Help" \
       label "     "\
	 widget HA -command "ha_p_c $arrayname"  label "       "  widget HA_REV label "revers heavy atom position  "
     bind $line.mb2 <Button-3>  "ShowMessage ha"
     bind $line.cb4 <Button-3>  "ShowMessage ha_rev"

    CloseSubFrame


    OpenSubFrame frame -toggle_display OASIS_MODE hide [list SIR DMR]

    OpenSubFrame frame -toggle_display X_RAY open [list X_WL]

       CreateLine line \
       message "Right click for Help" \
       label "     "\
	   widget HA -command "ha_p_c $arrayname"  label "       "  widget HA_REV label "revers heavy atom position  "\
     widget X_RAY label " "  widget X_WLENGTH 

     bind $line.mb2 <Button-3>  "ShowMessage ha"
     bind $line.cb4 <Button-3>  "ShowMessage ha_rev"

     bind $line.mb5 <Button-3>  "ShowMessage ha_wave"
     bind $line.e7 <Button-3>  "ShowMessage ha_wave_input"


    CloseSubFrame


    OpenSubFrame frame -toggle_display X_RAY hide [list X_WL]

       CreateLine line \
       message "Right click for Help" \
       label "     "\
	 widget HA -command "ha_p_c $arrayname"  label "       "  widget HA_REV label "revers heavy atom position  "\
    widget X_RAY 

     bind $line.mb2 <Button-3>  "ShowMessage ha"
     bind $line.cb4 <Button-3>  "ShowMessage ha_rev"
     bind $line.mb5 <Button-3>  "ShowMessage ha_wave"

    CloseSubFrame
    CloseSubFrame


      CreateInputFileLine line \
       "Right click for Help" \
        "              " \
        HA_FILE_PDB DIR_HA_FILE_PDB  \
         -command "ha_pdb $arrayname" \
       -toggle_display HA open [list HA_PDB]
     bind $line.mb2 <Button-3>  "ShowMessage ha_file_pdb"
     bind $line.e3 <Button-3>  "ShowMessage ha_file_pdb"


      CreateInputFileLine line \
       "Right click for Help" \
        "              " \
        HA_FILE_OAS DIR_HA_FILE_OAS \
         -command "ha_oas $arrayname" \
      -toggle_display HA open [list HA_OAS]
     bind $line.mb2 <Button-3>  "ShowMessage ha_file_frc"
     bind $line.e3 <Button-3>  "ShowMessage ha_file_frc"


    OpenSubFrame frame -toggle_display HA open HA_EN

    CreateExtendingFrame HA_T_NUM ha_num \
	   "Click to add an atom type and the number of independent atoms of that type"  \
	   "Add type"  \
	    [ list HA_T_TYPE HA_T_TYPE_NUM X_F HA_NUM_P ] \
         -child ha_p

   set ltmp  [winfo children [winfo parent $line]]
   set ltmp2 [winfo children  [lindex $ltmp end]]
   set ltmp3 [winfo children  [lindex $ltmp2 end]]
   set ltmp4  [lindex $ltmp3 end]
   bind $ltmp4.b1 <Button-3> "ShowMessage ha_atom_add"


    CloseSubFrame


    OpenSubFrame frame -toggle_display INIT hide 0
    OpenSubFrame frame -toggle_display HA hide [list HA_EN]
    OpenSubFrame frame -toggle_display HA_T_NUM open [list 1 2 3 4]

    OpenSubFrame frame -toggle_display X_RAY hide [list X_WL]
    CreateLine line \
    message "Right click for Help" \
    label "       Heavy atom: " widget HA_T_TYPE,1 \
    label " Number "  widget  HA_T_TYPE_NUM,1 \
    label " f'' "  widget   X_F,1 
    bind $line.e2 <Button-3>  "ShowMessage ha_type"
    bind $line.e4 <Button-3>  "ShowMessage ha_num"
    bind $line.e6 <Button-3>  "ShowMessage ha_f"
    CloseSubFrame

    OpenSubFrame frame -toggle_display X_RAY open [list X_WL]
    CreateLine line \
    message "Right click for Help" \
    label "       Heavy atom: " widget HA_T_TYPE,1 \
    label " Number "  widget  HA_T_TYPE_NUM,1 
    bind $line.e2 <Button-3>  "ShowMessage ha_type"
    bind $line.e4 <Button-3>  "ShowMessage ha_num"
    CloseSubFrame

    CloseSubFrame

    OpenSubFrame frame -toggle_display HA_T_NUM open [list 2 3 4]


    OpenSubFrame frame -toggle_display X_RAY hide [list X_WL]
    CreateLine line \
    message "Right click for Help" \
    label "       Heavy atom: " widget HA_T_TYPE,2 \
    label " Number "  widget  HA_T_TYPE_NUM,2 \
    label " f'' "  widget   X_F,2 \

    bind $line.e2 <Button-3>  "ShowMessage ha_type"
    bind $line.e4 <Button-3>  "ShowMessage ha_num"
    bind $line.e6 <Button-3>  "ShowMessage ha_f"
    CloseSubFrame

    OpenSubFrame frame -toggle_display X_RAY open [list X_WL]
    CreateLine line \
    message "Right click for Help" \
    label "       Heavy atom: " widget HA_T_TYPE,2 \
    label " Number "  widget  HA_T_TYPE_NUM,2 \

    bind $line.e2 <Button-3>  "ShowMessage ha_type"
    bind $line.e4 <Button-3>  "ShowMessage ha_num"
    CloseSubFrame


    CloseSubFrame

    OpenSubFrame frame -toggle_display HA_T_NUM open [list 3 4]
    OpenSubFrame frame -toggle_display X_RAY hide [list X_WL]
    CreateLine line \
    message "Right click for Help" \
    label "       Heavy atom: " widget HA_T_TYPE,3 \
    label " Number "  widget  HA_T_TYPE_NUM,3 \
    label " f'' "  widget   X_F,3 \

    bind $line.e2 <Button-3>  "ShowMessage ha_type"
    bind $line.e4 <Button-3>  "ShowMessage ha_num"
    bind $line.e6 <Button-3>  "ShowMessage ha_f"
    CloseSubFrame

    OpenSubFrame frame -toggle_display X_RAY open [list X_WL]
    CreateLine line \
    message "Right click for Help" \
    label "       Heavy atom: " widget HA_T_TYPE,3 \
    label " Number "  widget  HA_T_TYPE_NUM,3 \

    bind $line.e2 <Button-3>  "ShowMessage ha_type"
    bind $line.e4 <Button-3>  "ShowMessage ha_num"
    CloseSubFrame
	
    CloseSubFrame

    OpenSubFrame frame -toggle_display HA_T_NUM open [list 4]
    OpenSubFrame frame -toggle_display X_RAY hide [list X_WL]
    CreateLine line \
    message "Right click for Help" \
    label "       Heavy atom: " widget HA_T_TYPE,4 \
    label " Number "  widget  HA_T_TYPE_NUM,4 \
    label " f'' "  widget   X_F,4 \

    bind $line.e2 <Button-3>  "ShowMessage ha_type"
    bind $line.e4 <Button-3>  "ShowMessage ha_num"
    bind $line.e6 <Button-3>  "ShowMessage ha_f"
    CloseSubFrame

    OpenSubFrame frame -toggle_display X_RAY open [list X_WL]
    CreateLine line \
    message "Right click for Help" \
    label "       Heavy atom: " widget HA_T_TYPE,4 \
    label " Number "  widget  HA_T_TYPE_NUM,4 \

    bind $line.e2 <Button-3>  "ShowMessage ha_type"
    bind $line.e4 <Button-3>  "ShowMessage ha_num"

    CloseSubFrame

     CloseSubFrame
     CloseSubFrame
      CloseSubFrame

#-------------------------------------------------------------------------------------------
 OpenFolder 2  OASIS_MODE hide [list SAD SIR] closed 

     OpenSubFrame frame -toggle_display CYC_SIG open 0

     CreateLine line 

          CreateLine line \
              message "Right click for Help" \
              label "       Artificial heavy atoms randomly taking from" \
              widget NHA\
              label "% of the partial model"
           bind $line.e2 <Button-3>  "ShowMessage mr_nha"


          CreateLine line \
            message "Right click for Help" \
            label     "       Seed of random-number generator"\
            widget SED
           bind $line.e2 <Button-3>  "ShowMessage mr_seed"

    CloseSubFrame 

     OpenSubFrame frame -toggle_display CYC_SIG open 1

     CreateLine line \

          CreateLine line \
            message "Right click for Help" \
            label "       Artificial heavy atoms randomly taking from"\
            widget NHA\
            label "% of the partial model"
           bind $line.e2 <Button-3>  "ShowMessage mr_nha"


     CreateLine line \
            message "Right click for Help" \
            label "       Number of trials"\
            widget SED   
           bind $line.e2 <Button-3>  "ShowMessage mr_seed_num"


	OpenSubFrame frame -toggle_display FIRST_SIG open 0
     CreateLine line \
            message "Right click for Help" \
            label "         " widget FIRST_SIG -command "first_cyc_p $arrayname"  label "Special for first cycle" 
     bind $line.cb2 <Button-3>  "ShowMessage mr_first"

      CloseSubFrame 



	OpenSubFrame frame -toggle_display FIRST_SIG open 1
     CreateLine line \
            message "Right click for Help" \
            label "         " widget FIRST_SIG  label "Special at first cycle:" \
            label "Number of trials " widget first_seed \
            label " skip the first " widget skip_num label " trials"
     bind $line.cb2 <Button-3>  "ShowMessage mr_first" 
     bind $line.e4 <Button-3>  "ShowMessage mr_first_num"
     bind $line.e6 <Button-3>  "ShowMessage mr_first_skip"



      CloseSubFrame 

	OpenSubFrame frame -toggle_display BUILD_SIG open [list 1 3]
            CreateLine line \
            message "Right click for Help" \
            label "       Result selected from RESOLVE trials by "\
            widget MR_RESOLVE_SELECT

     bind $line.mb2 <Button-3>  "ShowMessage mr_select_resolve"


      CloseSubFrame 

	OpenSubFrame frame -toggle_display BUILD_SIG open [list 1 2]
            CreateLine line \
            label "       Result selected from ARP/wARP trials by "\
            widget MR_ARP_SELECT

            bind $line.mb2 <Button-3>  "ShowMessage mr_select_arp"


      CloseSubFrame 


      OpenSubFrame frame -toggle_display BUILD_SIG open [list 4]
            CreateLine line \
            label "       Result selected from PHENIX.AutoBuild trials by "\
            widget MR_PHENIX_SELECT
            bind $line.mb2 <Button-3>  "ShowMessage mr_select_phenix"

      CloseSubFrame 

    CloseSubFrame 



#-----------------------------------------------------------------------------------------

 OpenFolder 3
   
    OpenSubFrame frame -toggle_display CELL_CON open [list NUM]

       CreateLine line \
            message "Right click for Help" \
            widget CELL_CON \
            label "   "   \
            widget CELL_CON_NUM 

     bind $line.mb1 <Button-3>  "ShowMessage con"
     bind $line.e3 <Button-3>  "ShowMessage con_res_num"



    CreateExtendingFrame C_NUM C_other \
	    "Atoms not included in the heavy-atom substructure and other than C, N, O and H" \
	    "Other atoms" \
	    [ list C_TYPE C_TYPE_NUM ]

   set ltmp  [winfo children [winfo parent $line]]
   set ltmp2 [winfo children  [lindex $ltmp end]]
   set ltmp3  [lindex $ltmp2 end]
   bind $ltmp3.b1 <Button-3> "ShowMessage con_atom_add"



    CloseSubFrame

    OpenSubFrame frame -toggle_display CELL_CON open [list FILE]
    OpenSubFrame frame -toggle_display CYC_SIG open 0
       CreateLine line \
          widget CELL_CON 
          bind $line.mb1 <Button-3>  "ShowMessage con"
    CloseSubFrame


    OpenSubFrame frame -toggle_display CYC_SIG open 1
       CreateLine line \
          widget CELL_CON_CYC  -command "cell_p $arrayname"
          bind $line.mb1 <Button-3>  "ShowMessage con"
    CloseSubFrame


     CreateLine line \
            label "  Number of residues in the AU is "   \
            widget NMOL  \
		  label " times of that in the sequence file. " 

      bind $line.e2 <Button-3>  "ShowMessage con_au_num"

      CreateInputFileLine line \
        "Enter name of input pir file" \
        "           " \
        SEQIN DIR_SEQIN -command "is_pir_file $arrayname"

     bind $line.mb2 <Button-3>  "ShowMessage con_file_pir"
     bind $line.e3 <Button-3>  "ShowMessage con_file_pir"


    CloseSubFrame


    OpenSubFrame frame -toggle_display CELL_CON open [list OFILE]

    OpenSubFrame frame -toggle_display CYC_SIG open 0
       CreateLine line \
          widget CELL_CON            
          bind $line.mb1 <Button-3>  "ShowMessage con"
    CloseSubFrame


    OpenSubFrame frame -toggle_display CYC_SIG open 1
       CreateLine line \
          widget CELL_CON_CYC  -command "cell_p $arrayname"
          bind $line.mb1 <Button-3>  "ShowMessage con"
    CloseSubFrame


     CreateLine line \
            message "Right click for Help" \
            label "  Number of residues in the AU is "   \
            widget NMOL  \
                  label " times of that in the sequence file. "
      bind $line.e2 <Button-3>  "ShowMessage con_au_num"


      CreateInputFileLine line \
        "Right click for Help" \
        "           " \
        OSEQIN DIR_OSEQIN -command "is_pir_file $arrayname"

     bind $line.mb2 <Button-3>  "ShowMessage con_file_other"
     bind $line.e3 <Button-3>  "ShowMessage con_file_other"

    CloseSubFrame


#-----------------------------------------------------------------------------------------

 OpenFolder 4 closed


  CreateLine line \
            message "Right click for Help" \
            label "    " \
            widget RES \
            label "Resolution range from" \
            widget RES_L \
            label " to " \
            widget RES_H \
            label " Angstroms" 

     bind $line.cb2 <Button-3>  "ShowMessage resolution"
     bind $line.e3 <Button-3>  "ShowMessage resolution"
     bind $line.e5 <Button-3>  "ShowMessage resolution"

  CreateLine line \
      message "Right click for Help" \
      label "    " \
      widget COMPARE_PHI \
      label "Perform comparison with known phases from input file    "\
     
     bind $line.cb2 <Button-3>  "ShowMessage compare"


     CreateLabinLine line \
        "Known phases for comparison with those resulted from OASIS" \
        HKLIN "         PHIC" PHIC {NULL} \
        -toggle_display COMPARE_PHI open [list 1]

     bind $line.mb2 <Button-3>  "ShowMessage compare_file"


     CreateLine line \
        help kmi \
        message "Right click for Help" \
        label "       Kappa minimum" \
        widget KMI\
        message "Number of cycles for phase iteration (CYC keyword, 2 cycles recommended for SAD)" \
        label "     Number of cycles for phase iteration" \
        widget CYC

     bind $line.e2 <Button-3>  "ShowMessage kmi"
     bind $line.e4 <Button-3>  "ShowMessage cyc"


CreateLine line \
	    help cos \
	    message "Forcing cos(deltaPhi)'s to uniform distribution is for dealing with large experimental errors" \
     	    label "    " widget COS \
	    label "Not forcing cos(deltaPhi)'s to follow uniform distribution"\

     bind $line.cb2 <Button-3>  "ShowMessage cos"


#CreateLine line \
#            message "Forcing output FOM's to uniform distribution is for dealing with large experimental errors" \
#	    label "    " widget FOM \
#	    label "Not forcing output FOM's to follow uniform distribution"

#-----------------------------------------------------------------------------------------


 OpenFolder 5 BUILD_SIG hide [list 0 3 4] closed


  CreateLine line \
    message "Right click for Help" \
    label "       Target function"\
    message "Phased or nomral ML target function " \
    widget REF_MODE \
    label "  used in REFMAC5      "\
    message "Include or exclude and reflections flagged for Free R" \
    widget EXCLUDE_FREER \
    label " Use Free R flags"

     bind $line.mb2 <Button-3>  "ShowMessage arp_refine_mode"
     bind $line.cb4 <Button-3>  "ShowMessage arp_refine_rfree"


  CreateLine line \
    message "Right click for Help" \
        label "       Scaling mode "\
        widget SCALE 

     bind $line.mb2 <Button-3>  "ShowMessage arp_scale_mode"



  CreateLine line \
    message "Right click for Help" \    label "      " \
    widget IF_ARP7 \
    label "Use dipeptide distribution for Main-Chain tracing"

     bind $line.cb2 <Button-3>  "ShowMessage arp_if7"


  CreateLine line \
        message "Right click for Help" \        label "      " \
        widget IF_DUM  \
        label "Include dummy atoms in the structure model"

     bind $line.cb2 <Button-3>  "ShowMessage arp_rm_dum"


  CreateLine line \
    message "Right click for Help" \
    label "       Solution selected by "\
    widget ARP_SELECT

    bind $line.mb2 <Button-3>  "ShowMessage arp_select"


#-----------------------------------------------------------------------------------------
 OpenFolder 6 BUILD_SIG hide [list 0 1 3 2] closed

    OpenSubFrame frame -toggle_display autobuild_auto_signal hide fix

     CreateLine line  \
    message "Right click for Help" \
            label "      Build number" widget autobuild_auto_signal

    bind $line.mb2 <Button-3>  "ShowMessage autobuild_auto"


	CloseSubFrame    


    OpenSubFrame frame -toggle_display autobuild_auto_signal open fix

     CreateLine line  \
       message "Right click for Help" \
            label "      Build number" widget autobuild_auto_signal \
            label "  minimum builds" min_build widget min_build\
            label "  minimum rebuilds" widget min_rebuild

    bind $line.mb2 <Button-3>  "ShowMessage autobuild_auto"
    bind $line.e4 <Button-3>  "ShowMessage autobuild_build_min"
    bind $line.e6 <Button-3>  "ShowMessage autobuild_rebuild_min"



	CloseSubFrame    

     CreateLine line  \
            message "Right click for Help" \
            label "     " widget ONLY_SECOND \
            label "Secondary structure only"

    bind $line.cb2 <Button-3>  "ShowMessage autobuild_only_second"


  CreateLine line \
    message "Right click for Help" \
    label "       Solution selected by "\
    widget PHENIX_SELECT
    bind $line.mb2 <Button-3>  "ShowMessage autobuild_select"


}
#--------------------------------------------------------------------
proc mode_p { arrayname } {
#--------------------------------------------------------------------
  upvar #0 $arrayname array
  if { [GetValue $arrayname OASIS_MODE] == "SAD" } {
    set array(TITLE) "SAD phasing"
    set array(HA_SIG) 1
    set array(FOM) 1
    set array(COS) 0
    set array(PAR) 0
    set array(IF_DUM) 1
    SetValue $arrayname HL_TYPE "OASIS"
    set array(HL_control_signal) 1
  } elseif { [GetValue $arrayname OASIS_MODE] == "SIR" } {
    set array(TITLE) "SIR phasing"
    set array(HA_SIG) 1
    set array(FOM) 1
    set array(COS) 0
    set array(PAR) 0
    set array(IF_DUM) 1
    SetValue $arrayname HL_TYPE "OASIS"
    SetValue $arrayname X_RAY "X_WL"
    set array(HL_control_signal) 1
  } elseif { [GetValue $arrayname OASIS_MODE] == "DMR" } {
    set array(TITLE) "Partial-model extension"
    if { $array(SAD_TO_MR) } {
      set array(HA_SIG) 1	
    } else {
      set array(HA_SIG) 0
    }
    set array(FOM) 1
    set array(COS) 1
    set array(PAR) 1
    set array(IF_DUM) 0
    SetValue $arrayname HL_TYPE "OTHER"
    SetValue $arrayname X_RAY "X_WL"
    set array(HL_control_signal) 0
  }
}
#--------------------------------------------------------------------
proc ha_p { arrayname counter c4 } {
#--------------------------------------------------------------------
# Add another line to extending frame to enter details for heavy atom
# sites

    upvar #0 $arrayname array


    CreateLine line \
	    message "X, Y, Z - fractional coordinates" \
	    label "       X"    widget HA_X_T \
	    label "Y"    widget HA_Y_T \
	    label "Z"    widget HA_Z_T \
	    label "Occ"      widget HA_OCC_T\
	    label "B"        widget HA_B_T\
    

        bind $line.e2 <Button-3>  "ShowMessage ha_atom_coor"
        bind $line.e4 <Button-3>  "ShowMessage ha_atom_coor"
        bind $line.e6 <Button-3>  "ShowMessage ha_atom_coor"
        bind $line.e8 <Button-3>  "ShowMessage ha_atom_occ"
        bind $line.e10 <Button-3>  "ShowMessage ha_atom_b"

}

#--------------------------------------------------------------------
proc ha_c_p { arrayname } {
#--------------------------------------------------------------------


    upvar #0 $arrayname array

if { [GetValue $arrayname HA] == "HA_EN" } {
set array(HA_T_NUM) 0
 
     for {set i 1} { $i <= $array(HA_NUM) } {incr i} {
		set sig 0
           for {set j 1} { $j <= $array(HA_T_NUM) } {incr j} {
		if { $array(HA_T_TYPE,$j) == $array(HA_TYPE,$i) } {
			set sig $j
			break
}

}		
		if { $sig == 0}  {
		incr array(HA_T_NUM)
		set array(HA_T_TYPE,$array(HA_T_NUM)) $array(HA_TYPE,$i)  
		set array(HA_T_TYPE_NUM,$array(HA_T_NUM)) 1
} else {
		incr array(HA_T_TYPE_NUM,$sig) 
}		

}

}
}



#--------------------------------------------------------------------
proc ha_num { arrayname counter } {
#--------------------------------------------------------------------


    upvar #0 $arrayname array


    OpenSubFrame frame -toggle_display X_RAY hide [list X_WL]

    CreateLine line \
	    label "       Heavy atom " widget HA_T_TYPE \
	    label " Number"  widget  HA_T_TYPE_NUM  \
	    label "f''"   widget  X_F 

       bind $line.e2 <Button-3>  "ShowMessage ha_type"
       bind $line.e4 <Button-3>  "ShowMessage ha_num"
       bind $line.e6 <Button-3>  "ShowMessage ha_f"
   
    CloseSubFrame


    OpenSubFrame frame -toggle_display X_RAY open [list X_WL]

    CreateLine line \
	    label "       Heavy atom " widget HA_T_TYPE \
	    label " Number"  widget  HA_T_TYPE_NUM  \

        bind $line.e2 <Button-3>  "ShowMessage ha_type"
        bind $line.e4 <Button-3>  "ShowMessage ha_num"
    CloseSubFrame

   CreateExtendingFrame HA_NUM_P ha_p \
	   "Click to add a new heavy atom"  \
	   "Add atom"  \
	    [ list HA_X_T HA_Y_T HA_Z_T HA_OCC_T HA_B_T] \
          -counter $counter
}



#--------------------------------------------------------------------
proc C_other { arrayname counter } {
#--------------------------------------------------------------------


    upvar #0 $arrayname array

    CreateLine line \
	    label "       Type" widget C_TYPE \
	    label "       Number"  widget  C_TYPE_NUM 	

        bind $line.e2 <Button-3>  "ShowMessage con_type"
        bind $line.e4 <Button-3>  "ShowMessage con_type_num"    

}

#--------------------------------------------------------------------
proc merge_p { arrayname } {
#--------------------------------------------------------------------


    upvar #0 $arrayname array

    if {[GetValue $arrayname MER] == 1 } {
	set array(C_FOM) 0
}

}


#--------------------------------------------------------------------
proc fom_p { arrayname } {
#--------------------------------------------------------------------


    upvar #0 $arrayname array

    if {[GetValue $arrayname C_FOM] == 1 } {
	set array(MER) 0
}

}



#--------------------------------------------------------------------
proc map_p1 { arrayname } {
#--------------------------------------------------------------------


    upvar #0 $arrayname array
  if { [GetValue $arrayname PHASE_TYPE_SAD] == "oasis" } {
    if {[GetValue $arrayname MAP_FM] == 1 } {
      set array(MAP_FP) 0
    } else {
      set array(MAP_FM) 1
    }
  } else {
      set array(MAP_FM) 0
      set array(MAP_FP) 1


  }



}


#--------------------------------------------------------------------
proc map_p2 { arrayname } {
#--------------------------------------------------------------------


    upvar #0 $arrayname array

  if { [GetValue $arrayname PHASE_TYPE_SAD] == "oasis" } {
    if {[GetValue $arrayname MAP_FP] == 1 } {
	set array(MAP_FM) 0
    } else {
        set array(MAP_FP) 1
       }
    } else {
      set array(MAP_FM) 0
      set array(MAP_FP) 1
    }
}

#--------------------------------------------------------------------
proc mr_mode_p1 { arrayname } {
#--------------------------------------------------------------------


    upvar #0 $arrayname array

    if {[GetValue $arrayname MR_SAD] == 1 } {
	set array(MR_SIR) 0
} else {
        set array(MR_SIR) 1
}


}


#--------------------------------------------------------------------
proc mr_mode_p2 { arrayname } {
#--------------------------------------------------------------------


    upvar #0 $arrayname array

    if {[GetValue $arrayname MR_SIR] == 1 } {
	set array(MR_SAD) 0
} else {
        set array(MR_SAD) 1
}


}

#--------------------------------------------------------------------
proc ha_up { arrayname } {
#------------------------------------------------------------------

    upvar #0 $arrayname array

#       for {set i 1} { $i <= $array(HA_NUM) } {incr i } {
#       set array(HA_TYPE,$i) [string toupper $array(HA_TYPE,$i)]
#}
       for {set i 1} { $i <= $array(HA_T_NUM) } {incr i } {
#       set array(HA_T_TYPE,$i) [string toupper $array(HA_T_TYPE,$i)]
	if { [string toupper  $array(HA_T_TYPE,$i)] == "SE" } {
		set array(SE_SIG) 1
}
}

}


#--------------------------------------------------------------------
proc ha_pdb { arrayname } {
#------------------------------------------------------------------

upvar #0 $arrayname array

set array(INIT) 1
set array(HA_T_NUM) 0
set DIR_TMP [GetCurrentProjectDir $array(DIR_HA_FILE_PDB)]
append DIR_TMP "/$array(HA_FILE_PDB)"
           if { [ReadFile $DIR_TMP file_list -split ] }  {


              foreach line0 $file_list {
     	        set line [string trim $line0]
              set title  [lindex $line 0]

		if { $title == "ATOM" || $title == "HETATM" }  {
             set HA_TYPE   [lindex $line 2]

		set sig 0
              for {set j 1} { $j <= $array(HA_T_NUM) } {incr j} {
		 if { [string toupper $HA_TYPE] == [string toupper $array(HA_T_TYPE,$j)] } {
			set sig $j
			break
}
}		
		 if { $sig == 0}  {
		 incr array(HA_T_NUM)
           set len [string length $HA_TYPE]
           set c1 [string toupper [string range $HA_TYPE 0 0]]
           if { $len > 1 } {
             set c2 [string tolower [string range $HA_TYPE 1 $len]]
           } else {
             set c2 ""
           }
		 set array(HA_T_TYPE,$array(HA_T_NUM)) "$c1$c2" 
		 set array(HA_T_TYPE_ORI,$array(HA_T_NUM)) $HA_TYPE  
		 set array(HA_T_TYPE_NUM,$array(HA_T_NUM)) 1
} else {
		 incr array(HA_T_TYPE_NUM,$sig) 
}
}
}
}
}

#--------------------------------------------------------------------
proc ha_oas { arrayname } {
#------------------------------------------------------------------

upvar #0 $arrayname array
set array(INIT) 1
set array(HA_T_NUM) 0
set DIR_TMP [GetCurrentProjectDir $array(DIR_HA_FILE_OAS)]
append DIR_TMP "/$array(HA_FILE_OAS)"

           if { [ReadFile $DIR_TMP file_list -split ] }  {

              foreach line0 $file_list {
      	        set line [string trim $line0]
              set HA_TYPE   [lindex $line 0]

		if { $HA_TYPE == "" } {
		continue
}
		  set sig 0

              for {set j 1} { $j <= $array(HA_T_NUM) } {incr j} {
		 if { [string toupper $HA_TYPE] == [string toupper $array(HA_T_TYPE,$j)] } {
			set sig $j
			break
}
}		
		 if { $sig == 0}  {
		 incr array(HA_T_NUM)
         set len [string length $HA_TYPE]
           set c1 [string toupper [string range $HA_TYPE 0 0]]
         if { $len > 1 } {
           set c2 [string tolower [string range $HA_TYPE 1 $len]]
         } else {
           set c2 ""
         }
		 set array(HA_T_TYPE,$array(HA_T_NUM)) "$c1$c2"  
		 set array(HA_T_TYPE_ORI,$array(HA_T_NUM)) $HA_TYPE  
		 set array(HA_T_TYPE_NUM,$array(HA_T_NUM)) 1
} else {
		 incr array(HA_T_TYPE_NUM,$sig) 
}
}
}
}

#--------------------------------------------------------------------
proc ha_p_c { arrayname } {
#--------------------------------------------------------------------
# Add another line to extending frame to enter details for heavy atom
# sites

    upvar #0 $arrayname array
       for {set i 1} { $i <= $array(HA_T_NUM) } {incr i } {
       set array(HA_T_TYPE,$i) ""
       set array(HA_T_TYPE_NUM,$i) ""
}
if { [GetValue $arrayname HA ] != "HA_EN" } {
set array(INIT) 0
}
set array(HA_FILE_PDB) ""
set array(HA_FILE_OAS) ""

}
#-----------------------------------------------------------------------
proc c_up { arrayname } {
#------------------------------------------------------------------------
    upvar #0 $arrayname array

       for {set i 1} { $i <= $array(C_NUM) } {incr i } {
       set array(C_TYPE,$i) [string toupper $array(C_TYPE,$i)]
}
}
#-----------------------------------------------------------------------
proc dm_fom { arrayname } {
#------------------------------------------------------------------------
    upvar #0 $arrayname array

	if { $array(DM) == 1 && [GetValue $arrayname OASIS_MODE] == "DMR" } {
		if { $array(MOD) == 0 } {
			set array(MOD) 1
			SetValue $arrayname MOD_MOD "M_FOM"
			if { $array(M_HKLIN) == "" } {
			set array(M_HKLIN) $array(HKLIN)
			set array(DIR_M_HKLIN) $array(DIR_HKLIN)
}
}
}

}

#-----------------------------------------------------------------------
proc cyc_p { arrayname } {
#------------------------------------------------------------------------
    upvar #0 $arrayname array

	if { $array(CYC_SIG) == 1 } {

	if {  [GetValue $arrayname BUILD_TYPE] == "auto" } {
		set array(BUILD_SIG) 1
	} else {
		if { [GetValue $arrayname BUILD_TYPE] == "arp" } {
             set array(BUILD_SIG) 2
	} elseif { [GetValue $arrayname BUILD_TYPE] == "resolve" } {
		set array(BUILD_SIG) 3
	} else {
		set array(BUILD_SIG) 4
	}
	}
	if {[GetValue $arrayname OASIS_MODE] == "DMR" } {
		if { $array(MOD) == 0 } {
			set array(MOD) 1
			SetValue $arrayname MOD_MOD "M_FOM"
			if { $array(M_HKLIN) == "" } {
			set array(M_HKLIN) $array(HKLIN)
			set array(DIR_M_HKLIN) $array(DIR_HKLIN)
}
}
}
	SetValue $arrayname CELL_CON "FILE"
        SetValue $arrayname CELL_CON_CYC "FILE"
} else {
	set array(BUILD_SIG) 0
}
}

#-----------------------------------------------------------------------
proc in_p { arrayname } {
#------------------------------------------------------------------------
    upvar #0 $arrayname array
			set FOMM ""
             set array(AUTONAME) "oasis"

             set array(FREE) "Unassigned"

#puts $array(AUTONAME)
             set array(RES_L) $array(EXCLUDE_RESOLUTION_MIN)
             set array(RES_H) $array(EXCLUDE_RESOLUTION_MAX)
             if { $array(EXCLUDE_RESOLUTION_MAX) > 3.0 && $array(EXCLUDE_RESOLUTION_MIN) > 3.0 } {
            set array(ONLY_SECOND) 1
             } else {
            set array(ONLY_SECOND) 0
             }
			set array(M_HKLIN) $array(HKLIN)
			set array(DIR_M_HKLIN) $array(DIR_HKLIN)



}
#-----------------------------------------------------------------------
proc cell_p { arrayname } {
#------------------------------------------------------------------------
    upvar #0 $arrayname array
	SetValue $arrayname CELL_CON [GetValue $arrayname CELL_CON_CYC]
}

#-----------------------------------------------------------------------
proc build_type_p { arrayname } {
#------------------------------------------------------------------------
    upvar #0 $arrayname array

	if {  [GetValue $arrayname BUILD_TYPE] == "auto" } {
		set array(BUILD_SIG) 1
                SetValue $arrayname EXT_PRO "arp"
	} 
	if { [GetValue $arrayname BUILD_TYPE] == "arp" } {
             set array(BUILD_SIG) 2
                SetValue $arrayname EXT_PRO "arp"
	}
	if { [GetValue $arrayname BUILD_TYPE] == "resolve" } {
		set array(BUILD_SIG) 3
                SetValue $arrayname EXT_PRO "phenix"
	} 
	if { [GetValue $arrayname BUILD_TYPE] == "phenix" } {
		set array(BUILD_SIG) 4
                SetValue $arrayname EXT_PRO "phenix"
	}

}


#-----------------------------------------------------------------------
proc phase_type_p { arrayname } {
#------------------------------------------------------------------------
    upvar #0 $arrayname array

	if {   [GetValue $arrayname PHASE_TYPE_SAD] == "phaser" && [GetValue $arrayname OASIS_MODE] == "SAD" } {
		set array(MAP_FP) 1
		set array(MAP_FM) 0

	}
        if {   [GetValue $arrayname PHASE_TYPE_SAD] == "oasis" && [GetValue $arrayname OASIS_MODE] == "SAD" } {
                set array(MAP_FP) 0
                set array(MAP_FM) 1

        }
 
	if { [GetValue $arrayname PHASE_TYPE_MR] == "phaser" && [GetValue $arrayname OASIS_MODE] == "DMR" } {
	SetValue $arrayname PAR_MODE "P_PDB"
}
	if { [GetValue $arrayname OASIS_MODE] == "SIR" } {
	SetValue $arrayname PHASE_TYPE_SAD "oasis"
}
}

#-----------------------------------------------------------------------
proc par_p { arrayname } {
#------------------------------------------------------------------------
    upvar #0 $arrayname array
  if { [GetValue $arrayname OASIS_MODE] == "DMR" } {
    set array(PAR) 1	
  }

  if { $array(PAR) == 0 } {
     SetValue $arrayname FOM_TYPE "OASIS"
  } 

}


#-----------------------------------------------------------------------
proc par_format_p { arrayname } {
#------------------------------------------------------------------------
    upvar #0 $arrayname array
	if { ([GetValue $arrayname PHASE_TYPE_MR] == "phaser" || [GetValue $arrayname EXT_MODEL] ) && [GetValue $arrayname OASIS_MODE] == "DMR" } {
	SetValue $arrayname PAR_MODE "P_PDB"
}

}

#-----------------------------------------------------------------------
proc fom_type_p { arrayname } {
#------------------------------------------------------------------------
    upvar #0 $arrayname array


}

#-----------------------------------------------------------------------
proc sad_to_mr_p { arrayname } {
#------------------------------------------------------------------------
    upvar #0 $arrayname array
	if { $array(SAD_TO_MR) } {
		set array(HA_SIG) 1	
		} else {
		set array(HA_SIG) 0
		}
}


#-----------------------------------------------------------------------
proc first_cyc_p { arrayname } {
#------------------------------------------------------------------------
    upvar #0 $arrayname array
    set  array(first_seed) $array(SED) 
}



#------------------------------------------------------------------------
proc ShowMessage { keyw args } {
#------------------------------------------------------------------------


  global configure
  global warning_message_flag

  set title "Help"
  set w ".help"
  set help_file ""
  set button_text "Close"
  set stop 0
  set transient 0
  set command "show_message_command chooseopt dismiss"
  set centre ""
  set nograb 1
  set image_file ""
  set font_name ""
  set color ""

  set bgcolor "#ffffff"
  set path_doc [GetEnvPath CHTML]

  append path_doc "/oasis_help"
  if { [file exists $path_doc/$keyw.dat]  } {
     if {![ReadFile  $path_doc/$keyw.dat message]} {
       WarningMessage "Can't open the help file."
       return 0
     }
  } else {
     WarningMessage "No Help is available."
     return 0
  }
  # If there is already a warning message up then just add in the extra message

  set n 0
  while {[winfo exists $w.main.canvas.contents.text_$n]} { incr n }

  if { $n == 0 } {
    OpenWindow $w  $title $title -help $help_file 
    CreateFrame $w chooseopt -noscroll
#    CreateButton $w dismiss $button_text "$command"
#    CreateButton $w edit "edit" "exec /Applications/TextEdit.app/Contents/MacOS/TextEdit  $path_doc/$keyw.dat & "
    # Define behaviour which should occur when the window manager is used
    # to delete the window
    wm protocol $w WM_DELETE_WINDOW "$command"
  }

  # Set up the font
  if { $font_name == "" } {
    # Default font
    if { [string length $message ] > 200 } {
      set font  $configure(FONT_SMALL)
    } else {
      set font $configure(FONT_REGULAR)
    }
  } else {
    # User defined font
    # See the Tk font command for information on font families etc
    set font $font_name
  }
  $w configure -bg $bgcolor
  $w.buttons configure -bg $bgcolor
  set tw  [label $w.main.canvas.contents.text_$n  \
        -wraplength "15 c" \
        -justify left \
        -font $font \
        -text "$message" ]
  pack $tw -side top -fill x

  # Colour the text?
  $tw configure -bg $bgcolor

  update idletasks
  wm deiconify $w
  raise $w

  if { $n > 0 } { return }

  if { $centre != "" } {
    CentreWindow $w $centre
  } else {
    CentreWindow $w SCREEN
  }
  if { $nograb == 0 }  {
    catch "grab $w"
    add_modalDialog_safeguard $w
  }
  if { $stop } { 
    vwait warning_message_flag
  } elseif { $transient} {
    set delay [ expr {1000 *     \
     [ lindex $args [expr {[lsearch $args "-transient" ] + 1} ] ]} ]
    after $delay "$command"
  }

}


#---------------------------------------------------------------------
proc show_message_command { arrayname returntext } {
#---------------------------------------------------------------------

  upvar #0 $arrayname array
  global warning_message_flag
  catch "grab release $array(WINDOW)"
  CloseWindow $arrayname
  update idletasks
  set warning_message_flag $returntext
}




#--------------------------------------------------------------------
proc is_pir_file { arrayname } {
#--------------------------------------------------------------------

    upvar #0 $arrayname array

  if { [GetValue $arrayname CELL_CON] == "FILE" } {
    set filen [GetFullFileName1 $array(SEQIN) $array(DIR_SEQIN) ]
  } elseif { [GetValue $arrayname CELL_CON] == "OFILE" } {
    set filen [GetFullFileName1 $array(OSEQIN) $array(DIR_OSEQIN) ]
  } 
  # Read file contents into a list split on newlines
  if { ![ReadFile $filen file_text -split] } {
    WarningMessage "Could not open file!"
    return 0
  }

  # Must have at least 3 lines in the file I guess
  set nlines [llength $file_text]
  if { $nlines < 3 } {
    WarningMessage "Not enough lines in .pir file!"
    return 0
  }

  # Start to check the format

  # First line starts with a ">" and with any trailing text
  set line [lindex $file_text 0]
  if { ![regexp -- "^>" $line] } {
    WarningMessage "First line of .pir file doesn't start with \">\"!"
    return 0
  }

  # Second line is empty - white spaces allowed
  set line [lindex $file_text 1]
  if { ![regexp -- "^\[ \t\]*$" $line] } {
    WarningMessage "Second line of .pir file is not empty!"
    return 0

  }

  # Remaining lines can only contain one or more of the characters
  # ACDEFGHIKLMNQPRSTYWV
#  for { set i 2 } { $i < $nlines } { incr i } {
#      set line [lindex $file_text $i]
#      if {![regexp -- "^\[ACDEFGHIKLMNQPRSTYWV\]+$" $line]} {
#         puts "Non-standard amino acid code in .pir file! $line"
#         return 0
#      }
#  }
  # Reached end of file successfully, so this matches
  # our description of .pir file
  return 1
}

