#!/bin/csh -f

if ( $?CCP4 == 0 ) then
  echo "CCP4 hasn't been installed, you should install it before OASIS"
   goto end;
endif

if ( $?warpbin == 0 ) then
   echo -n "ARP/wARP hasn't been installed, this may cause problems in model building. Do you want to continue? [y or n, default n]"
   set confirm = $<
   if ( $confirm != "y" && $confirm != "Y" ) goto end;
endif


if ( $?PHENIX == 0 ) then
   echo -n "PHENIX hasn't been installed, this may cause problems in using RESOLVE, AutoBuild and PyMOL. Do you want to continue? [y or n, default n]:"
   set confirm = $<
   if ( $confirm != "y" && $confirm != "Y" ) goto end;
endif


if (  !(-e ./bin) || !(-e ./ccp4i) ) then
  echo "Some necessary files are missing, please check!"
  goto end;
endif



if ( $?oasisbin == 1 ) then
   echo -n "You have installed OASIS previously, do you want to overwrite the existent directory? [y or n, default y]:"
   set path_confirm = $<
   set d_path = `cd $oasisbin; cd .. ; pwd`
else
   set path_confirm = "n"
   set d_path = "$CBIN/oasis_bin"
endif

set install_path = ""

if ( $path_confirm != "n" && $path_confirm != "N" ) then
  set install_path = $d_path
else
   echo -n "Input install path [default: $d_path]:"
   set install_path = $<

   if ( $install_path == "" ) then
     set install_path = $d_path
   endif

endif

if ( !(-e $install_path) ) then
   echo -n "The destination doesn't exist, do you want to create it? [y or n, default y]:"
   set creat_path = $<
   if ( $creat_path == "n" && $creat_path != "N" ) goto end
   echo `mkdir $install_path`
     if ( !(-e $install_path) ) then
        echo "can't create oasis path, please check."
        goto end
     endif
else
    if ( -e "$install_path/bin" && `pwd` != $install_path ) then
      set m_path = `cd $install_path ; pwd`
      set Date = `date +%Y%m%d`
      rm -fr $m_path/oasisbackup_$Date
      mkdir $m_path/oasisbackup_$Date
      mv $install_path/bin $install_path/oasisbackup_$Date
    endif
endif

set CCP4_DIR = ${CCP4}
set CCP4i = ${CCP4I_TOP}
set CCP4_MOD = ${CCP4i}/etc/UNIX/modules.def
set CCP4_TMP = ${CCP4i}/etc/UNIX/temp.def
set CCP4_LIST = ${CCP4I_HELP}/modules/programlist.html
set CCP4_LIST_TMP = ${CCP4I_HELP}/modules/TEM.html
set CCP4_EX = ${CCP4I_HELP}/modules/exptphase.html
set CCP4_INDEX = ${CHTML}/INDEX.html
set CCP4_INDEX_TMP = ${CHTML}/INDEX_TMP.html





rm -f ${CBIN}/oasis_old
mv ${CBIN}/oasis ${CBIN}/oasis_old
cp ./bin/oasis4-0 ${CBIN}/oasis



sed '/MODULE_TASK_LIST/ s/Oasis - SAD\/SIR phasing/Oasis - SAD\/SIR\/Partial-model extension/' $CCP4_MOD > $CCP4_TMP 
rm -f $CCP4_MOD
mv $CCP4_TMP $CCP4_MOD 

sed '/TASK_TITLE/ s/Oasis - SAD\/SIR phasing/Oasis - direct methods Phasing/' $CCP4_MOD > $CCP4_TMP
rm -f $CCP4_MOD
mv $CCP4_TMP $CCP4_MOD


sed '/FOLDER_TASK_LIST/ s/Oasis - SAD\/SIR phasing/Oasis - direct methods Phasing/' $CCP4_MOD > $CCP4_TMP
rm -f $CCP4_MOD
mv $CCP4_TMP $CCP4_MOD


sed '/TASK_DESCRIPT/ s/Use Oasis to break the phase ambiguity for SAD or SIR/Oasis - direct methods of SAD\/SIR phasing and Partial-model extension/' $CCP4_MOD > $CCP4_TMP


rm -f $CCP4_MOD
mv $CCP4_TMP $CCP4_MOD

sed 's/exptphase.html#oasis/..\/..\/..\/html\/oasis_gui\/oasis.html/' $CCP4_LIST > $CCP4_LIST_TMP
rm -f $CCP4_LIST
mv $CCP4_LIST_TMP $CCP4_LIST

sed 's/..\/..\/..\/html\/oasis.html/..\/..\/..\/html\/oasis_gui\/oasis.html/' $CCP4_LIST > $CCP4_LIST_TMP
rm -f $CCP4_LIST
mv $CCP4_LIST_TMP $CCP4_LIST

sed 's/Oasis - SAD\/SIR phasing/Oasis - direct methods of SAD\/SIR phasing and Partial-model extension/' $CCP4_LIST > $CCP4_LIST_TMP

rm -f $CCP4_LIST
mv $CCP4_LIST_TMP $CCP4_LIST

sed 's/See <a href="..\/..\/..\/html\/oasis.html" target="_PROGDOC">OASIS/<a href="..\/..\/..\/html\/oasis_gui\/index.html" target="_PROGDOC">OASIS tutorial/' $CCP4_LIST > $CCP4_LIST_TMP

rm -f $CCP4_LIST
mv $CCP4_LIST_TMP $CCP4_LIST

sed 's/See <a href="..\/..\/..\/html\/oasis\/oasis.html" target="_PROGDOC">OASIS/<a href="..\/..\/..\/html\/oasis_gui\/index.html" target="_PROGDOC">OASIS tutorial/' $CCP4_LIST > $CCP4_LIST_TMP

rm -f $CCP4_LIST
mv $CCP4_LIST_TMP $CCP4_LIST


sed 's/Oasis - SAD\/SIR phasing/Oasis - direct methods of SAD\/SIR phasing and Partial-model extension/g' $CCP4_EX >$CCP4_LIST_TMP

rm -f $CCP4_EX
mv $CCP4_LIST_TMP $CCP4_EX


sed 's/<li><a href="oasis.html">oasis<\/a> - A program for breaking phase ambiguity in OAS or SIR <\/li>/<li><a href="oasis_gui\/oasis.html">oasis<\/a> - direct methods of SAD\/SIR phasing and Partial-model extension<\/li>/g' $CCP4_INDEX >$CCP4_INDEX_TMP

rm -f $CCP4_INDEX
mv $CCP4_INDEX_TMP $CCP4_INDEX


rm -fr ${CHTML}/oasis_gui
cp -r ./user_guide ${CHTML}/oasis_gui




if ( -e ${CHTML}/oasis_help ) then
  rm -fr  ${CHTML}/oasis_help
endif
cp -r ./ccp4i/oasis_help ${CHTML}/

rm -f ${CCP4i}/tasks/oasis_old.tcl 
mv ${CCP4i}/tasks/oasis.tcl ${CCP4i}/tasks/oasis_old.tcl
cp ./ccp4i/oasis.tcl ${CCP4i}/tasks/oasis.tcl

rm -f ${CCP4i}/tasks/oasis_old.def
mv ${CCP4i}/tasks/oasis.def ${CCP4i}/tasks/oasis_old.def
cp ./ccp4i/oasis.def ${CCP4i}/tasks/oasis.def


if ( -e ${CCP4i}/templates/oasis.com ) then
mv ${CCP4i}/templates/oasis.com ${CCP4i}/templates/oasis_old.com
endif

rm -f ${CCP4i}/scripts/oasis_old.script
mv ${CCP4i}/scripts/oasis.script ${CCP4i}/scripts/oasis_old.script
cp ./ccp4i/oasis.script ${CCP4i}/scripts/oasis.script


if ( $path_confirm != "n" && $path_confirm != "N" ) then
  set c_path = `pwd`
  if ( "$c_path" != $install_path ) then
    if ( -e $install_path/bin ) then
      set m_path = `cd $install_path ; pwd`
      set Date = `date +%Y%m%d`
      rm -fr $m_path/oasisbackup_$Date
      mkdir $m_path/oasisbackup_$Date
      mv $install_path/bin/* $install_path/oasisbackup_$Date
      cp -R ./bin/* $install_path/bin
    else
      mkdir $install_path/bin
      cp -R ./bin/* $install_path/bin
    endif
  endif
else
#
	echo
	echo
	echo
	echo ' OASIS has been installed '
	echo ' ==============================================================='
	echo ' For tcsh/csh:'
	echo ' make sure that the following line is in your .tcshrc/.cshrc file:'
	echo "" setenv oasisbin \"$install_path/bin\" 
	echo ' For bash:'
	echo ' make sure that the following line is in your .bashrc file:'
	echo "" export oasisbin=\"$install_path/bin\" 
	echo
	echo ' ==============================================================='
     if (`pwd` != $install_path) then
       mkdir $install_path/bin
       cp -R ./bin/* $install_path/bin
     endif
endif

exit
end:
echo "Install failed!"
exit

